import { TmTextareaElement } from 'src';

/**
 * Provides key bindings for tab-based indentation in textareas. This includes both single line and multi-line indentation.
 * The function adds event listeners to the passed element for handling 'keydown' and 'input' events specific to indentation.
 *
 * @param element - The textarea element to which indentation functionality will be applied.
 *
 * @example
 * import { TabIndentation } from "tm-textarea/bindings"
 *
 * return <tm-textarea ref={TabIndentation} />
 */
export declare const TabIndentation: {
    binding(element: HTMLTextAreaElement | TmTextareaElement): () => void;
    /**
     * Handles keydown events for indenting and outdenting lines in a textarea.
     * It triggers an 'input' event with types 'formatIndent' or 'formatOutdent'
     * based on whether the tab was pressed with the shift key.
     *
     * @param event - The keyboard event triggered when a key is pressed.
     */
    onKeyDown(event: KeyboardEvent & {
        currentTarget: TmTextareaElement | HTMLTextAreaElement;
    }): void;
    /**
     * Handles 'input' events specifically for processing 'formatIndent' and 'formatOutdent' input types.
     * Modifies the textarea's content based on the type of indentation required.
     *
     * @param e - The input event that was dispatched during indentation handling.
     */
    onInput(event: InputEvent & {
        currentTarget: TmTextareaElement | HTMLTextAreaElement;
    }): void;
    outdent(source: string, tabSize: number): string;
    indent(source: string): string;
    getLeadingWhitespace(source: string): string;
    getLineStart(value: string, position: number): number;
    /**
     * Calculates the whitespace segments for a string of leading whitespace, merging certain segments for visual consistency.
     *
     * This function is designed to normalize the leading whitespace into consistent tab or space segments. It ensures that partial
     * tab-sized segments of spaces are merged into single tabs or combined to fit the defined tab size, aiding in consistent indentation handling.
     *
     * @param leadingWhitespace - The string of leading whitespace from a line of text.
     * @param tabSize - The number of spaces that constitute a tab segment.
     * @returns {string[]} - An array of strings, each representing a coherent segment of indentation.
     */
    getIndentationSegments(leadingWhitespace: string, tabSize: number): string[];
    /**
     * Formats the indentation of each line in a given source string using tabs.
     * It calculates the amount of leading whitespace in each line and replaces it with tabs based on the specified tab size.
     *
     * @param source - The string of text to format.
     * @param tabSize - The number of spaces that represent a single tabulation in the context of the source text.
     * @returns The source text with spaces replaced by tabs as per the calculated indentation levels.
     */
    format(source: string, tabSize: number): string;
};
